package cn.com.ty.lift.batch.applet.service;

import cn.com.ty.lift.batch.applet.constants.DataStatisticsConstants;
import cn.com.ty.lift.batch.applet.dao.mapper.EmergencyRepairDataMapper;
import cn.com.ty.lift.batch.applet.dao.model.EmergencyRepairDataModel;
import cn.com.ty.lift.batch.applet.dao.model.request.CommonRequest;
import cn.com.ty.lift.batch.applet.dao.model.request.EmergencyRepairDataRequest;
import cn.com.xwy.boot.web.dto.RestResponse;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author huangyuan
 * @date 2020/2/3
 * @description
 */
@Service
public class EmergencyRepairDataService {

    @Resource
    private EmergencyRepairDataMapper emergencyRepairDataMapper;

    @Resource
    private RedisTemplate redisTemplate;

    /**
     * @param commonRequest 起始时间，结束时间
     * @return
     * @description 将指定的急修数据放入redis中
     * @date 2020/2/3 3:33 下午
     */
    public void putEmergencyRepairDataListToRedis(CommonRequest commonRequest) {
        //获取时间段内的急修数据
        List<EmergencyRepairDataModel> emergencyRepairDataModelList = emergencyRepairDataMapper
                .getEmergencyRepairDataList(commonRequest);
        //通过公司id将数据进行分组
        //将急修时间段内的数据放入redis缓存中
        redisTemplate.opsForValue().set(DataStatisticsConstants.EMERGENCY_REPAIR_DATA_FIELD, emergencyRepairDataModelList);
    }

    /**
     * @param emergencyRepairDataRequest 起始时间，结束时间
     * @return
     * @description 获取年度急修数据
     * @date 2020/2/3 4:28 下午
     */
    public RestResponse getEachMonthData(EmergencyRepairDataRequest emergencyRepairDataRequest) {
        List<EmergencyRepairDataModel> emergencyRepairDataModelList = (List<EmergencyRepairDataModel>) redisTemplate
                .opsForValue().get(DataStatisticsConstants.EMERGENCY_REPAIR_DATA_FIELD);
        if (emergencyRepairDataRequest.getStartTimeDate() != null) {
            //过滤起始时间之前的数据
            emergencyRepairDataModelList.stream()
                    .filter(a -> a.getCallerDate().isAfter(emergencyRepairDataRequest.getStartTimeDate()));
        }
        if (emergencyRepairDataRequest.getEndTimeDate() != null) {
            //过滤终止时间之后的数据
            emergencyRepairDataModelList.stream()
                    .filter(a -> a.getCallerDate().isBefore(emergencyRepairDataRequest.getEndTimeDate()));
        }
        return RestResponse.success();
    }

}
