package cn.com.ty.lift.business.emergency.controller;


import cn.com.ty.lift.business.emergency.dto.LiftFaultCategory;
import cn.com.ty.lift.business.emergency.dto.LiftFaultType;
import cn.com.ty.lift.business.emergency.dto.RepairRequest;
import cn.com.ty.lift.business.emergency.dto.RepairResponse;
import cn.com.ty.lift.business.emergency.entity.EmergencyRepair;
import cn.com.ty.lift.business.emergency.entity.ErRecordCost;
import cn.com.ty.lift.business.emergency.entity.ErRecordImg;
import cn.com.ty.lift.business.emergency.entity.LiftFault;
import cn.com.ty.lift.business.emergency.service.IEmergencyRepairService;
import cn.com.ty.lift.business.emergency.service.IErRecordCostService;
import cn.com.ty.lift.business.emergency.service.IErRecordImgService;
import cn.com.ty.lift.business.emergency.service.ILiftFaultService;
import cn.com.ty.lift.business.evaluation.dao.entity.Evaluation;
import cn.com.ty.lift.business.evaluation.service.EvaluationService;
import cn.com.ty.lift.common.model.CountPage;
import cn.com.ty.lift.common.utils.Judge;
import cn.com.ty.lift.common.utils.Judger;
import cn.com.ty.lift.common.utils.StrPool;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.IterUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *     临修记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/emergency/emergency-repair")
public class EmergencyRepairController {

    private IEmergencyRepairService emergencyRepairService;
    private IErRecordCostService    erRecordCostService;
    private IErRecordImgService     erRecordImgService;
    private ILiftFaultService       liftFaultService;
    private EvaluationService       evaluationService;

    /**
     * 根据id查找
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("findOne")
    @Judger(fields = {"id"})
    public RestResponse findOne(@RequestBody RepairRequest request){
        Long id = request.getId();
        RepairResponse result = emergencyRepairService.selectByIdWithInfo(request);

        if(null != result){
            //急修费用
            result.setErRecordCosts(erRecordCostService.listByErRecordId(id));
            //急修图片
            result.setErRecordImgs(erRecordImgService.listByErRecordId(id));
            //评价
            result.setEvaluation(evaluationService.findByRecord(id));
            //故障信息
            fillFault(result);
        }
        return RestResponse.success(result);
    }

    /**
     * 解析故障项id字符串，查询出相关的故障项
     * @param result RepairResponse
     */
    private void fillFault(RepairResponse result){
        List<Long> partIds = splitIds(result.getFaultPart());
        List<Long> reasonIds = splitIds(result.getFaultReason());
        List<Long> natureIds = splitIds(result.getFaultNature());
        List<Long> handleIds = splitIds(result.getFaultHandle());
        List<Long> dutyIds = splitIds(result.getFaultDuty());
        List<Long> faultIds = new ArrayList<>();
        faultIds.addAll(partIds);
        faultIds.addAll(reasonIds);
        faultIds.addAll(natureIds);
        faultIds.addAll(handleIds);
        faultIds.addAll(dutyIds);
        Collection<LiftFault> liftFaults = liftFaultService.listByIds(faultIds);
        if(IterUtil.isNotEmpty(liftFaults)){
            result.setFaultParts(liftFaults.stream().filter(lf -> (partIds.contains(lf.getId()))).collect(Collectors.toList()));
            result.setFaultReasons(liftFaults.stream().filter(lf -> (reasonIds.contains(lf.getId()))).collect(Collectors.toList()));
            result.setFaultNatures(liftFaults.stream().filter(lf -> (natureIds.contains(lf.getId()))).collect(Collectors.toList()));
            result.setFaultHandles(liftFaults.stream().filter(lf -> (handleIds.contains(lf.getId()))).collect(Collectors.toList()));
            result.setFaultDuties(liftFaults.stream().filter(lf -> (dutyIds.contains(lf.getId()))).collect(Collectors.toList()));
        }
    }

    /**
     * 拆分故障项id字符串，组装成List<Long>
     * @param idStr ids
     * @return List<Long>
     */
    private List<Long> splitIds(String idStr){
        if(StrUtil.isEmpty(idStr)){
            return Collections.EMPTY_LIST;
        }
        List<Long> ids = Arrays.asList(idStr.split(StrPool.COMMA)).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
        return ids;
    }
    /**
     * 新增急修
     * @param entity EmergencyRepair
     * @return RestResponse
     */
    @PostMapping("add")
    public RestResponse add(@Valid @RequestBody EmergencyRepair entity){
        Long mtCompanyId = entity.getMtCompanyId();
        Long liftId = entity.getLiftId();
        int repairing = emergencyRepairService.countRepairing(mtCompanyId,liftId);
        Judge.nlt0(repairing,Judge.Repair.haveUnfinisedRepair);

        //派单时间 急修单创建时间
        entity.setHasEvaluate(0);
        entity.setAssignTime(LocalDateTime.now());
        entity.setStatus(Judge.Repair.STATUS_TO_REPAIRED);
        boolean result = emergencyRepairService.save(entity);
        return RestResponse.success(result);
    }

    /**
     * 分页查询急修中的 status = 2
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("list")
    @Judger(fields = "mtCompanyId")
    public RestResponse list(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByRepairing(request);
        long count = pages.getTotal();
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * 根据条件分页查询
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("page")
    @Judger(fields = "mtCompanyId")
    public RestResponse page(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByCondition(request);
        long count = emergencyRepairService.countByCondition(request);
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     *  根据条件分页查询
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("pageWorker")
    @Judger(fields = {"workerId"})
    public RestResponse pageWorker(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByCondition(request);
        return RestResponse.success(pages);
    }

    /**
     * 急修转派
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("transfer")
    @Judger(fields = {"id","workerId"})
    public RestResponse transfer(@RequestBody RepairRequest request){
        EmergencyRepair entity = emergencyRepairService.getById(request.getId());
        Judge.notNull(entity);

        //如果已经接单，在急修中了不能被转派
        Judge.notTrue(Judge.Repair.statusRepairing(entity.getStatus()),Judge.Repair.repairHadTaked);
        //本急修单的负责维保工
        entity.setWorkerId1(request.getWorkerId());

        boolean result = emergencyRepairService.saveOrUpdate(entity);
        return RestResponse.success(result);
    }

    /**
     * 关闭急修
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("close")
    @Judger(fields = {"id","closeReason"})
    public RestResponse close(@RequestBody RepairRequest request){
        EmergencyRepair entity = emergencyRepairService.getById(request.getId());
        Judge.notNull(entity);

        entity.setCloseReason(request.getCloseReason());
        //新状态（-1 暂停中，0 待修理；1 修理中；2 已完成 3 已关闭）
        entity.setStatus(Judge.Repair.STATUS_CLOSE);

        boolean result = emergencyRepairService.saveOrUpdate(entity);
        return RestResponse.success(result);
    }

    /**
     * app-其他角色获取自己关联的项目的急修任务
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("pageByUser")
    @Judger(fields = {"userId"})
    public RestResponse pageByUser(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByUser(request);
        return RestResponse.success(pages);
    }

    /**
     * app-点击接单，保存接单时间到急修表的接单时间接口
     * 接单的时候就是签到
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("taking")
    @Judger(fields = {"id","takingTime"})
    public RestResponse taking(@RequestBody RepairRequest request){
        EmergencyRepair entity = emergencyRepairService.getById(request.getId());
        Judge.notNull(entity);

        Judge.isTrue(Judge.Repair.statusToRepaired(entity.getStatus()),Judge.Repair.mustToRepair);
        entity.setTakingTime(request.getTakingTime());
        //todo:修改电梯的业务状态为急修中,如果存在维保中，要修改
        entity.setStatus(Judge.Repair.STATUS_REPAIRING);

        boolean result = emergencyRepairService.taking(entity);
        return RestResponse.success(result);
    }

    /**
     * app-点击到达，保存到达时间到急修表的到达时间中
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("arrive")
    @Judger(fields = {"id","arriveTime","position"})
    public RestResponse arrive(@RequestBody RepairRequest request){
        EmergencyRepair entity = emergencyRepairService.getById(request.getId());
        Judge.notNull(entity);

        Judge.isTrue(Judge.Repair.statusRepairing(entity.getStatus()), Judge.Repair.mustInRepairing);
        entity.setArriveTime(request.getArriveTime());
        entity.setPosition(request.getPosition());

        boolean result = emergencyRepairService.updateById(entity);
        return RestResponse.success(result);
    }

    /**
     * app-停梯时间+安全确认，保存停梯时间+安全确认项到急修表中
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("stop")
    @Judger(fields = {"id","stopDate","safetyConfirm"})
    public RestResponse stopLift(@RequestBody RepairRequest request){
        EmergencyRepair entity = emergencyRepairService.getById(request.getId());
        Judge.notNull(entity);

        Judge.isTrue(Judge.Repair.statusRepairing(entity.getStatus()),Judge.Repair.mustInRepairing);
        entity.setStopDate(request.getStopDate());
        entity.setSafetyConfirm(request.getSafetyConfirm());

        boolean result = emergencyRepairService.updateById(entity);
        return RestResponse.success(result);
    }

    /**
     * 新增急修收费单项
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("addCost")
    @Judger(fields = {"id","erRecordCosts"})
    public RestResponse addCost(@RequestBody RepairRequest request){
        EmergencyRepair repair = emergencyRepairService.getById(request.getId());
        Judge.notNull(repair);

        List<ErRecordCost> erRecordCosts = request.getErRecordCosts();
        BigDecimal costTotal = erRecordCosts.stream().map(ErRecordCost::getCostMoney).reduce(BigDecimal.ZERO,(m1,m2) -> m1.add(m2));
        repair.setCostTotal(costTotal);

        boolean result = emergencyRepairService.addCost(repair, erRecordCosts);
        return RestResponse.success(result);
    }

    /**
     * 删除急修下的收费项
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("deleteCost")
    @Judger(fields = {"id","erCostItemId"})
    public RestResponse deleteCost(@RequestBody RepairRequest request){
        Long id = request.getId();
        Long erCostItemId = request.getErCostItemId();

        EmergencyRepair repair = emergencyRepairService.getById(id);
        Judge.notNull(repair);

        ErRecordCost erRecordCost = erRecordCostService.findByRecordAndId(id,erCostItemId);
        Judge.notNull(erRecordCost);

        BigDecimal costTotal = repair.getCostTotal();
        BigDecimal costMoney = erRecordCost.getCostMoney();
        Judge.notNull(costTotal,Judge.Repair.incorrectAmount);
        Judge.notNull(costMoney,Judge.Repair.incorrectAmount);
        //收费项的金额不能大于急修单的总金额
        Judge.notTrue(costTotal.compareTo(costMoney) == -1,Judge.Repair.incorrectAmount);

        repair.setCostTotal(costTotal.subtract(costMoney));

        boolean result = emergencyRepairService.deleteCost(repair, erRecordCost);
        return RestResponse.success(result);
    }

    /**
     * 修改急修单下的急修收费项
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("modifyCost")
    @Judger(fields = {"id","erRecordCosts"})
    public RestResponse modifyCost(@RequestBody RepairRequest request){
        EmergencyRepair repair = emergencyRepairService.getById(request.getId());
        Judge.notNull(repair);

        List<ErRecordCost> erRecordCosts = request.getErRecordCosts();
        BigDecimal costTotal = erRecordCosts.stream().map(ErRecordCost::getCostMoney).reduce(BigDecimal.ZERO,(m1,m2) -> m1.add(m2));
        repair.setCostTotal(costTotal);

        boolean result = emergencyRepairService.modifyCost(repair, erRecordCosts);
        return RestResponse.success(result);
    }

    /**
     * 根据电梯类型查询故障信息（集合结构）
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("fault/list")
    @Judger(fields = {"liftCategory"})
    public RestResponse faultList(@RequestBody RepairRequest request){
        List<LiftFault> lists = liftFaultService.listByLiftCategory(request.getLiftCategory());
        return RestResponse.success(lists);
    }

    /**
     * 根据电梯类型获取急修故障信息列表(树结构)
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("fault/tree")
    @Judger(fields = {"liftCategory"})
    public RestResponse faultTree(@RequestBody RepairRequest request) {
        List<LiftFault> liftFaultList = liftFaultService.listByLiftCategory(request.getLiftCategory());
        Judge.notNull(liftFaultList,Judge.Repair.missingFault);
        //根据故障列表组装tree
        //先按照faultCategory，groupby,再按照faultType，groupby
        Map<String, Map<String,List<LiftFault>>> categories = liftFaultList.stream().collect(Collectors.groupingBy(LiftFault::getFaultCategory, Collectors.groupingBy(LiftFault::getFaultType)));
        List<LiftFaultCategory> liftFaultCategories = new ArrayList<>();
        if(MapUtil.isNotEmpty(categories)){
            categories.forEach((category,faultTypes) -> {
                List<LiftFaultType> liftFaultTypes = new ArrayList<>();
                faultTypes.forEach((type,liftFaults) -> {
                    liftFaultTypes.add(new LiftFaultType(type,liftFaults));
                });
                liftFaultCategories.add(new LiftFaultCategory(category,liftFaultTypes));
            });
        }
        return RestResponse.success(liftFaultCategories);
    }

    /**
     * 急修评价
     * @param entity 封装评价的参数model
     * @return RestResponse
     */
    @PostMapping("evaluate")
    public RestResponse evaluate(@Valid @RequestBody Evaluation entity){
        Long recordId = entity.getRecordId();
        EmergencyRepair repair = emergencyRepairService.getById(recordId);
        Judge.notNull(repair);

        Judge.isTrue(Judge.Repair.statusFinish(repair.getStatus()),Judge.Repair.mustFinish);
        Judge.notTrue(repair.getHasEvaluate() == 1,Judge.Repair.hadEvaluate);

        //来源（1 维保；2 急修）
        entity.setSource(2);
        entity.setLiftId(repair.getLiftId());
        entity.setPostDate(LocalDateTime.now());

        //设置评价状态
        repair.setHasEvaluate(1);

        boolean result = emergencyRepairService.evaluate(repair,entity);
        return RestResponse.success(result);
    }

    /**
     * 生成急修单，添加主要，次要急修人签名
     * @param request 请求参数model
     * @return RestResponse
     */
    @PostMapping("repairOrder")
    @Judger(fields = {"id","recoveryDate","mainSign","erRecordImg","fieldDescription",
            "faultPart","faultReason","faultHandle","faultNature","faultDuty"})
    public RestResponse repairOrder(@RequestBody RepairRequest request){
        Long id = request.getId();
        List<String> erRecordImg = request.getErRecordImg();

        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        Judge.isTrue(Judge.Repair.statusRepairing(entity.getStatus()),Judge.Repair.mustInRepairing);
        List<ErRecordImg> erRecordImgs = new ArrayList<>();
        for (int i = 0; i < erRecordImg.size(); i++) {
            ErRecordImg item = new ErRecordImg();
            item.setErRecordId(id);
            item.setImgUrl(erRecordImg.get(i));
            item.setSort(i);
            erRecordImgs.add(item);
        }
        entity.setMainSign(request.getMainSign());
        entity.setSecondSign(request.getSecondSign());
        entity.setFieldDescription(request.getFieldDescription());
        entity.setFaultPart(request.getFaultPart());
        entity.setFaultReason(request.getFaultReason());
        entity.setFaultHandle(request.getFaultHandle());
        entity.setFaultNature(request.getFaultNature());
        entity.setFaultDuty(request.getFaultDuty());
        entity.setRecoveryDate(request.getRecoveryDate());
        //修改状态已完成
        entity.setStatus(Judge.Repair.STATUS_FINISH);

        boolean result = emergencyRepairService.repairOrder(entity,erRecordImgs);
        return RestResponse.success(result);
    }

    /**
     * 查询急修的急修图片
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("listImages")
    @Judger(fields = {"id"})
    public RestResponse listImages(@RequestBody RepairRequest request){
        List<ErRecordImg> lists = erRecordImgService.listByErRecordId(request.getId());
        Judge.notNull(lists);
        return RestResponse.success(lists);
    }

    /**
     * 电梯管理中查询急修记录
     * @param request RepairRequest
     * @return RestResponse
     */
    @PostMapping("listRepair")
    @Judger(fields = {"mtCompanyId","liftId","projectId"})
    public RestResponse listRepair(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByCompanyLiftProject(request);
        return RestResponse.success(pages);
    }
}
