package cn.com.ty.lift.business.framework.conf;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.time.LocalDateTime;
import java.util.Objects;

/**
 * @author bieao
 * @date 2019/12/23
 * @description 公共字段配置
 */
@Component
public class MetaHandler implements MetaObjectHandler {

    private static final String CREATOR_ID = "creatorId";
    private static final String CREATE_DATE = "createDate";
    private static final String UPDATE_ID = "updateId";
    private static final String UPDATE_DATE = "updateDate";

    @Override
    public void insertFill(MetaObject metaObject) {
        LocalDateTime now = LocalDateTime.now();
        //获取当前登陆用户id
        Long userId = getUserId();
        fill(CREATOR_ID, userId, metaObject);
        fill(CREATE_DATE, now, metaObject);
        fill(UPDATE_ID, userId, metaObject);
        fill(UPDATE_DATE, now, metaObject);
    }

    @Override
    public void updateFill(MetaObject metaObject) {
        //获取当前登陆用户id
        Long userId = getUserId();
        LocalDateTime now = LocalDateTime.now();
        fill(UPDATE_ID, userId, metaObject);
        fill(UPDATE_DATE, now, metaObject);
    }

    private Long getUserId() {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
                .getRequest();
        HttpSession session = request.getSession();
        Object userId = session.getAttribute(ApiConstants.CURRENT_USER_ID);
        if (ObjectUtil.isNotEmpty(userId)) {
            return Long.parseLong(Objects.toString(userId));
        }
        return 10000L;
    }

    /**
     * 填充公共属性的值
     *
     * @param fieldName  属性名
     * @param fieldVal   属性值
     * @param metaObject
     */
    private void fill(String fieldName, Object fieldVal, MetaObject metaObject) {
        //检查原有的值，setter方法
        Object value = getFieldValByName(fieldName,metaObject);
        if (ObjectUtil.isEmpty(value) && metaObject.hasSetter(fieldName)) {
            setFieldValByName(fieldName, fieldVal, metaObject);
        }
    }
}
