package cn.com.ty.lift.business.framework.job;

import cn.com.ty.lift.business.maintenance.dao.entity.MaintenancePlan;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.hutool.core.collection.IterUtil;
import cn.hutool.core.date.DateUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author wcz
 * @Description: 更新状态job
 * @date 2020/1/18 17:16
 */
@Slf4j
@Component
@EnableScheduling
@AllArgsConstructor
public class StatusUpdateJob {

    private MaintenancePlanService maintenancePlanService;
    /**
     * 更新维保计划过期状态job：每天1-3点，每10分钟执行一次
     */
    @Scheduled(cron = "0 0/10 1-3 * * ?")
    public void updateMtPlanJob(){
        log.info("updateMtPlanJob : {}",DateUtil.now());
        List<MaintenancePlan> plans = maintenancePlanService.listOverduePlan(1000);
        if(IterUtil.isNotEmpty(plans)){
            plans.forEach(plan ->{
                //全部设置成超期
                plan.setStatus(2);
            });
            maintenancePlanService.updateBatchById(plans);
        }
    }
}
