package cn.com.ty.lift.business.project.controller;

import cn.com.ty.lift.business.capital.dao.entity.SignIn;
import cn.com.ty.lift.business.capital.service.SignInService;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftExtensionRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceRecord;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceAddRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceAppRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceRecordDetailRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenancePlanResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenanceRecordResponse;
import cn.com.ty.lift.business.maintenance.service.MaintenanceRecordService;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectAppUser;
import cn.com.ty.lift.business.project.dao.entity.model.request.CompanyRequest;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectDetailRequest;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectCompanyResponse;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectDetailResponse;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectLiftResponse;
import cn.com.ty.lift.business.project.service.ProjectAppService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * @author bieao
 * @date 2019/12/31
 * @description app端控制层(项目、电梯、维保、大修)
 */
@RestController
@RequestMapping("app")
public class ProjectAppController {

    @Resource
    private ProjectAppService projectAppService;

    @Resource
    private MaintenanceRecordService maintenanceRecordService;

    @Resource
    private SignInService signInService;

    /**
     * @param request 公司id
     * @return RestResponse 项目列表
     * @description 根据公司id查询公司下项目列表
     * @date 2019/12/31 3:13 PM
     */
    @PostMapping("project/list")
    public RestResponse list(@Valid @RequestBody CompanyRequest request) {
        IPage<ProjectCompanyResponse> page = projectAppService.list(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 项目id
     * @return RestResponse 维保工列表
     * @description 根据项目id查询项目下维保工
     * @date 2020/1/14 11:25 上午
     */
    @PostMapping("project/user/list")
    public RestResponse userList(@Valid @RequestBody ProjectDetailRequest request) {
        List<ProjectAppUser> userList = projectAppService.userList(request.getId());
        if (CollUtil.isEmpty(userList)) {
            return RestResponse.success();
        }
        return RestResponse.success(userList, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request app端新增电梯
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增电梯
     * @date 2020/1/11 4:21 下午
     */
    @PostMapping("lift/add")
    public RestResponse add(@RequestBody LiftExtensionRequest request) {
        return projectAppService.add(request);
    }

    /**
     * @param request app端保养信息
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增保养记录
     * @date 2020/1/13 2:12 下午
     */
    @PostMapping("record/add")
    public RestResponse add(@RequestBody MaintenanceAddRecordRequest request) {
        return maintenanceRecordService.saveRecord(request);
    }

    /**
     * @param record app端保养信息
     * @return 1.成功, 0.失败, 消息描述
     * @description 编辑保养记录
     * @date 2020/1/13 2:33 下午
     */
    @PostMapping("record/modify")
    public RestResponse modify(@RequestBody MaintenanceRecord record){
        boolean result = maintenanceRecordService.modify(record);
        if (result) {
            return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
        } else {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
    }

    /**
     * @param request 保养id
     * @return RestResponse 保养记录
     * @description 查看保养记录
     * @date 2020/1/13 2:33 下午
     */
    @PostMapping("record/detail")
    public RestResponse detail(@RequestBody MaintenanceRecordDetailRequest request){
        MaintenanceRecord record = maintenanceRecordService.detail(request.getId());
        if (ObjectUtil.isEmpty(record)) {
            return RestResponse.success();
        }
        return RestResponse.success(record, MessageUtils.get("msg.query.detail.success"));
    }

    /**
     * @param request 项目id
     * @return RestResponse 电梯列表
     * @description 根据项目id查询项目下电梯列表
     * @date 2019/12/31 3:13 PM
     */
    @PostMapping("lift/list")
    public RestResponse list(@Valid @RequestBody ProjectDetailRequest request) {
        IPage<ProjectLiftResponse> page = projectAppService.list(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 项目id
     * @return RestResponse 项目详情
     * @description 根据项目id查询项目详情
     * @date 2020/1/2 1:50 下午
     */
    @PostMapping("project/detail")
    public RestResponse detail(@Valid @RequestBody ProjectDetailRequest request) {
        ProjectDetailResponse response = projectAppService.detail(request);
        if (ObjectUtil.isEmpty(response)) {
            return RestResponse.success();
        }
        return RestResponse.success(response, MessageUtils.get("msg.query.detail.success"));
    }

    /**
     * @param request 用户id和维保状态
     * @return RestResponse 保养任务列表
     * @description 根据用户id和维保状态查询保养任务列表
     * @date 2020/1/6 11:29 上午
     */
    @PostMapping("maintenance/task")
    public RestResponse taskList(@Valid @RequestBody MaintenanceAppRequest request) {
        IPage<MaintenancePlanResponse> response = projectAppService.taskList(request);
        if (CollUtil.isEmpty(response.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(response, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 公司id和电梯id
     * @return RestResponse 保养信息
     * @description 根据公司id和电梯id查询上次保养信息
     * @date 2020/1/6 5:37 下午
     */
    @PostMapping("maintenance/record/info")
    public RestResponse recordInfo(@Valid @RequestBody MaintenanceRecordRequest request) {
        MaintenanceRecordResponse response = maintenanceRecordService.recordInfo(request);
        if (ObjectUtil.isEmpty(response)) {
            return RestResponse.success();
        }
        return RestResponse.success(response, MessageUtils.get("msg.query.detail.success"));
    }

    /**
     * @param entry 用户签到信息
     * @return 1.成功, 0.失败, 消息描述
     * @description 大修签到
     * @date 2020/2/5 12:45 下午
     */
    @PostMapping("capital/sign/in")
    public RestResponse signIn(@RequestBody SignIn entry){
        boolean result = signInService.add(entry);
        if (result) {
            return RestResponse.success(null, MessageUtils.get("msg.add.success"));
        } else {
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
    }
}
