package cn.com.ty.lift.enterprise.custom.service.impl;

import cn.com.ty.lift.enterprise.custom.dao.entity.PropertyCompany;
import cn.com.ty.lift.enterprise.custom.dao.entity.model.PropertyCompanyAndPropertyContactReq;
import cn.com.ty.lift.enterprise.custom.dao.entity.model.PropertyCompanyReq;
import cn.com.ty.lift.enterprise.custom.dao.mapper.PropertyCompanyMapper;
import cn.com.ty.lift.enterprise.custom.service.PropertyCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.poi.excel.BigExcelWriter;
import cn.hutool.poi.excel.ExcelUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * (甲方公司)客户表 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-11-11
 */
@Service
@AllArgsConstructor
public class PropertyCompanyServiceImpl extends ServiceImpl<PropertyCompanyMapper, PropertyCompany> implements PropertyCompanyService {
    private final PropertyContactServiceImpl propertyContactService;

    public RestResponse propertyCompanies(PropertyCompanyReq req) {
        Page<PropertyCompany> page = new Page<>(req.getPageNum(), req.getPageSize());
        //根据请求参数查询客户列表
        IPage<PropertyCompany> records = this
                .page(page, new QueryWrapper<PropertyCompany>()
                        .select("id", "mt_company_id", "name", "mailing_address", "status")
                        .eq(req.getMtCompanyId() != null, "mt_company_id", req.getMtCompanyId())
                        .ne("status", 0)
                        .and(req.getInfo() != null, c ->
                                c
                                        .like("name", req.getInfo())
                                        .or()
                                        .like("address", req.getInfo())
                                        .or()
                                        .like("mailing_address", req.getInfo())
                        ));
        if (records.getRecords().isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(records, "成功");
    }

    @Transactional
    public RestResponse saveAll(PropertyCompanyAndPropertyContactReq req) {
        long id = IdWorker.getId();
        req.getPropertyCompany().setId(id);
        req.getPropertyContacts().forEach(contact -> contact.setPpCompanyId(id));
        if (this.save(req.getPropertyCompany()) && propertyContactService.saveBatch(req.getPropertyContacts())) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }

    @Transactional
    public RestResponse delete(Long id) {
        PropertyCompany byId = this.getById(id);
        if (null == byId) {
            return RestResponse.success();
        }
        if (byId.getStatus() == 2) {
            return RestResponse.success(null, "有正在服务的关联项目，无法删除");
        } else if (baseMapper.selectProjectCount(byId.getId()) != null
                && baseMapper.selectProjectCount(byId.getId()).get("count") > 0L) {
            byId.setStatus(2);
            this.updateById(byId);
            return RestResponse.success(null, "有正在服务的关联项目，无法删除");
        } else {
            byId.setStatus(0);
            if (this.updateById(byId)) {
                return RestResponse.success(null, "成功");
            }
            return RestResponse.fail();
        }
    }

    public void export(PropertyCompany propertyCompany, HttpServletResponse res) {
        String fileName = "甲方公司_" + LocalDate.now().toString() + ".xlsx";
        List<PropertyCompany> companies = this.list(new QueryWrapper<PropertyCompany>()
                .select("id", "name", "mailing_address", "status")
                .eq("mt_company_id", propertyCompany.getMtCompanyId())
                .ne("status", 0));

        List<List<?>> rows = new ArrayList<>();
        //设置列头
        rows.add(CollUtil.newArrayList("甲方公司id", "名称", "通讯地址", "状态,1正常 2有关联项目"));
        companies.forEach(c -> {
            List<?> row = CollUtil.newArrayList(c.getId().toString(), c.getName(), c.getMailingAddress(), c.getStatus());
            rows.add(row);
        });
        BigExcelWriter writer = ExcelUtil.getBigWriter(System.getProperty("user.dir") + "/" + fileName);
        //设置列宽
        writer.setColumnWidth(0, 25);
        writer.setColumnWidth(1, 35);
        writer.setColumnWidth(2, 40);
        writer.setColumnWidth(3, 25);
        // 一次性写出内容，使用默认样式
        writer.write(rows);
        // 关闭writer，释放内存
        writer.close();

        res.setHeader("Content-Disposition", "attachment; filename=" + fileName);
        res.setHeader("content-type", "application/x-msdownload;");
        res.setContentType("text/plain; charset=utf-8");
        byte[] buff = new byte[1024];
        BufferedInputStream bis = null;
        OutputStream os = null;

        File file = new File(System.getProperty("user.dir") + "/" + fileName);
        try {
            os = res.getOutputStream();
            bis = new BufferedInputStream(new FileInputStream(file));
            int i = bis.read(buff);

            while (i != -1) {
                os.write(buff, 0, buff.length);
                os.flush();
                i = bis.read(buff);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (os != null) {
                try {
                    os.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        if (file.exists()) {
            file.delete();
        }
    }
}