package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.common.model.CountPage;
import cn.com.ty.lift.common.utils.Judger;
import cn.com.ty.lift.enterprise.oa.dto.AnnouncementRequest;
import cn.com.ty.lift.enterprise.oa.dto.AnnouncementResponse;
import cn.com.ty.lift.enterprise.oa.entity.Announcement;
import cn.com.ty.lift.enterprise.oa.service.IAnnouncementImgService;
import cn.com.ty.lift.enterprise.oa.service.IAnnouncementService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;


/**
 * <p>
 *     公司公告 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */

@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/oa/announcement")
public class AnnouncementController {

    private IAnnouncementService    announcementService;
    private IAnnouncementImgService announcementImgService;

    /**
     * 根据id查找公告
     * @param request AnnouncementRequest
     * @return RestResponse
     */
    @PostMapping("findOne")
    @Judger(fields = {"id"})
    public RestResponse findOne(@RequestBody AnnouncementRequest request) {
        AnnouncementResponse result = announcementService.findByIdWithInfo(request);
        if(null != result){
            result.setImgs(announcementImgService.listByAnnouncement(request.getId()));
        }
        return RestResponse.success(result);
    }

    /**
     * 根据条件分页查找公告
     * @param request AnnouncementRequest
     * @return RestResponse
     */
    @PostMapping("list")
    @Judger(fields = "mtCompanyId")
    public RestResponse list(@RequestBody AnnouncementRequest request) {
        IPage<AnnouncementResponse> pages = announcementService.pageByCondition(request);
        long count = announcementService.count();
        CountPage<AnnouncementResponse> countPage = new CountPage<>(count, pages);
        return RestResponse.success(countPage);
    }

    /**
     * 添加公告
     * @param entity Announcement
     * @return RestResponse
     */
    @PostMapping("add")
    public RestResponse add(@Valid @RequestBody Announcement entity) {
        boolean result = announcementService.add(entity);
        return RestResponse.success(result);
    }

    /**
     * 根据id删除公告
     * @param request AnnouncementRequest
     * @return RestResponse
     */
    @PostMapping("delete")
    @Judger(fields = {"id"})
    public RestResponse delete(@RequestBody AnnouncementRequest request) {
        boolean result = announcementService.removeById(request.getId());
        return RestResponse.success(result);

    }
}
