package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.common.model.CountPage;
import cn.com.ty.lift.common.utils.Judge;
import cn.com.ty.lift.common.utils.Judger;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.entity.MaintenanceCompany;
import cn.com.ty.lift.enterprise.oa.service.IAttendanceImgService;
import cn.com.ty.lift.enterprise.oa.service.IAttendanceService;
import cn.com.ty.lift.enterprise.oa.service.IMaintenanceCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.time.LocalTime;

/**
 * <p>
 *      打卡签到记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/oa/attendance")
public class AttendanceController {

    private IAttendanceService         attendanceService;
    private IMaintenanceCompanyService maintenanceCompanyService;
    private IAttendanceImgService      attendanceImgService;

    /**
     * 根据id查找
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("findOne")
    @Judger(fields = {"id"})
    public RestResponse findOne(@RequestBody AttendanceRequest request) {
        AttendanceResponse result = attendanceService.findByIdWithInfo(request);
        if(null != result){
            result.setImgs(attendanceImgService.listByAttendance(request.getId()));
        }
        return RestResponse.success(result);
    }

    /**
     * 根据条件查询
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("list")
    @Judger(fields = "mtCompanyId")
    public RestResponse list(@RequestBody AttendanceRequest request) {
        IPage<AttendanceResponse> pages = attendanceService.pageByCondition(request);
        long count = attendanceService.count();
        CountPage<AttendanceResponse> countPage = new CountPage<>(count, pages);
        return RestResponse.success(countPage);
    }

    /**
     * 设置打卡时间
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("setupTime")
    @Judger(fields = {"mtCompanyId"})
    public RestResponse setupTime(@RequestBody AttendanceRequest request) {
        LocalTime topTime = request.getTopTime();
        LocalTime downTime = request.getDownTime();
        Judge.notTrue(ObjectUtil.hasEmpty(topTime, downTime), Judge.Attend.SetAmTime);
        Judge.notTrue(topTime.isAfter(downTime), Judge.Attend.InvalidAmTime);

        LocalTime topTimePm = request.getTopTimePm();
        LocalTime downTimePm = request.getDownTimePm();
        Judge.notTrue(ObjectUtil.hasEmpty(topTimePm, downTimePm), Judge.Attend.SetPmTime);
        Judge.notTrue(topTimePm.isAfter(downTimePm), Judge.Attend.InvalidPmTime);

        LocalTime topTimeOt = request.getTopTimeOt();
        LocalTime downTimeOt = request.getDownTimeOt();
        Judge.notTrue(ObjectUtil.hasEmpty(topTimeOt, downTimeOt), Judge.Attend.SetOtTime);
        Judge.notTrue(topTimeOt.isAfter(downTimeOt), Judge.Attend.InvalidOtTime);

        MaintenanceCompany entity = maintenanceCompanyService.getById(request.getMtCompanyId());
        Judge.notNull(entity);

        entity.setTopTime(topTime);
        entity.setDownTime(downTime);
        entity.setTopTimePm(topTimePm);
        entity.setDownTimePm(downTimePm);
        entity.setTopTimeOt(topTimeOt);
        entity.setDownTimeOt(downTimeOt);

        boolean result = maintenanceCompanyService.saveOrUpdate(entity);
        return RestResponse.success(result);
    }

    /**
     * 打卡
     * 11 上午上班 12 上午下班
     * 21 下午上班 22 下午下班
     * 31 加班上班 32 加班下班
     *
     * @param entity Attendance
     * @return RestResponse
     */
    @PostMapping("clock")
    public RestResponse clock(@Valid @RequestBody Attendance entity) {
        Long mtCompanyId = entity.getMtCompanyId();
        Long userId = entity.getUserId();
        Integer type = entity.getType();
        //先统计当天是否有对应的打卡记录
        int count = attendanceService.countByUserAndType(mtCompanyId,userId,type);
        Judge.lt0(count,Judge.Attend.hadClock);
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(mtCompanyId);
        Judge.notNull(maintenanceCompany);

        LocalTime now = LocalTime.now();
        //状态（是否迟到早退，0：否，1：是）
        switch (type) {
            case 11:
                LocalTime topTime = maintenanceCompany.getTopTime();
                Judge.notNull(topTime, Judge.Attend.SetAmTime);
                //打卡时间比上班时间早-正常，否则迟到
                entity.setStatus(now.isBefore(topTime) ? 0 : 1);
                entity.setRemarks(Judge.Attend.AmTopClock);
                break;
            case 12:
                LocalTime downTime = maintenanceCompany.getDownTime();
                Judge.notNull(downTime, Judge.Attend.SetAmTime);
                //打卡时间比下班时间晚-正常，否则早退
                entity.setStatus(now.isAfter(downTime) ? 0 : 1);
                entity.setRemarks(Judge.Attend.AmDownClock);
                break;
            case 21:
                LocalTime topTimePm = maintenanceCompany.getTopTimePm();
                Judge.notNull(topTimePm, Judge.Attend.SetPmTime);
                entity.setStatus(now.isBefore(topTimePm) ? 0 : 1);
                entity.setRemarks(Judge.Attend.PmTopClock);
                break;
            case 22:
                LocalTime downTimePm = maintenanceCompany.getDownTimePm();
                Judge.notNull(downTimePm, Judge.Attend.SetPmTime);
                entity.setStatus(now.isAfter(downTimePm) ? 0 : 1);
                entity.setRemarks(Judge.Attend.PmDownClock);
                break;
            case 31:
                LocalTime topTimeOt = maintenanceCompany.getTopTimeOt();
                Judge.notNull(topTimeOt, Judge.Attend.SetOtTime);
                entity.setStatus(now.isBefore(topTimeOt) ? 0 : 1);
                entity.setRemarks(Judge.Attend.OtTopClock);
                break;
            case 32:
                LocalTime downTimeOt = maintenanceCompany.getDownTimeOt();
                Judge.notNull(downTimeOt, Judge.Attend.SetOtTime);
                entity.setStatus(now.isAfter(downTimeOt) ? 0 : 1);
                entity.setRemarks(Judge.Attend.OtDownClock);
                break;
            default:
                return RestResponse.failParam();
        }
        boolean result = attendanceService.save(entity);
        return RestResponse.success(result);
    }

    /**
     * 返回维保公司打卡时间信息
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("time")
    @Judger(fields = {"mtCompanyId"})
    public RestResponse time(@RequestBody AttendanceRequest request) {
        MaintenanceCompany entity = maintenanceCompanyService.getById(request.getMtCompanyId());
        return RestResponse.success(entity);
    }

    /**
     * 根据用户查询打卡记录
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("pageByUser")
    @Judger(fields = {"userId"})
    public RestResponse pageByUser(@RequestBody AttendanceRequest request) {
        IPage<AttendanceResponse> pages = attendanceService.pageByUser(request);
        return RestResponse.success(pages);
    }
}
