package cn.com.ty.lift.enterprise.oa.service.impl;

import cn.com.ty.lift.enterprise.oa.dto.AnnouncementRequest;
import cn.com.ty.lift.enterprise.oa.dto.AnnouncementResponse;
import cn.com.ty.lift.enterprise.oa.entity.Announcement;
import cn.com.ty.lift.enterprise.oa.entity.AnnouncementImg;
import cn.com.ty.lift.enterprise.oa.mapper.AnnouncementMapper;
import cn.com.ty.lift.enterprise.oa.service.IAnnouncementService;
import cn.hutool.core.collection.IterUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 公司公告 服务类
 * </p>
 *
 * @author wcz
 * @since 2019-12-12
 */
@Service
@AllArgsConstructor
public class AnnouncementService extends ServiceImpl<AnnouncementMapper,Announcement> implements IAnnouncementService {

    private AnnouncementImgService announcementImgService;
    /**
     * 根据条件分页查询公告记录
     * @param request AnnouncementRequest
     * @return IPage<AnnouncementResponse>
     */
    public IPage<AnnouncementResponse> pageByCondition(AnnouncementRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<AnnouncementResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByCondition(page,request);
    }

    /**
     * 根据id查询，带全部关联信息
     * @param request AnnouncementRequest
     * @return AnnouncementResponse
     */
    public AnnouncementResponse findByIdWithInfo(AnnouncementRequest request){
        return baseMapper.findByIdWithInfo(request);
    }

    /**
     * 新增公告，同时插入图片
     * @param entity Announcement
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean add(Announcement entity){
        boolean an = save(entity);
        //插入公告失败，直接返回false
        if(!an){
            return false;
        }
        //没有图片，直接返回true
        List<String> imgUrls = entity.getImgUrls();
        if(IterUtil.isEmpty(imgUrls)){
            return true;
        }
        //循环处理，批量插入图片
        List<AnnouncementImg> imgList = new ArrayList<>();
        for (int i = 0; i < imgUrls.size(); i++) {
            AnnouncementImg ai = new AnnouncementImg();
            ai.setAnnouncementId(entity.getId());
            ai.setImgUrl(imgUrls.get(i));
            ai.setSort(i);
            imgList.add(ai);
        }
        boolean ais = announcementImgService.saveBatch(imgList);
        if(ais){
            return true;
        }else{
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }
}