package cn.com.ty.lift.enterprise.oa.service.impl;

import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.mapper.AttendanceMapper;
import cn.com.ty.lift.enterprise.oa.service.IAttendanceService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;

/**
 * <p>
 * 打卡签到记录 服务类
 * </p>
 *
 * @author wcz
 * @since 2019-12-12
 */
@Service
public class AttendanceService extends ServiceImpl<AttendanceMapper,Attendance> implements IAttendanceService {

    /**
     * 根据条件分页查找签到记录
     * @param request AttendanceRequest
     * @return IPage<AttendanceResponse>
     */
    public IPage<AttendanceResponse> pageByCondition(AttendanceRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<AttendanceResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByCondition(page, request);
    }

    /**
     * 根据用户分页查找记录
     * @param request AttendanceRequest
     * @return IPage<AttendanceResponse>
     */
    public IPage<AttendanceResponse> pageByUser(AttendanceRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<AttendanceResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByUser(page, request);
    }

    /**
     * 根据用户和打卡类型统计打卡记录条数
     * @param mtCompanyId 维保公司id
     * @param userId 用户id
     * @param type 打卡类型
     * @return int
     */
    public int countByUserAndType(Long mtCompanyId,Long userId,Integer type){
        LambdaQueryWrapper<Attendance> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(Attendance::getMtCompanyId,mtCompanyId);
        lambdaQueryWrapper.eq(Attendance::getUserId,userId);
        lambdaQueryWrapper.eq(Attendance::getType,type);
        //查询当天的打卡记录
        LocalDateTime begin = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
        LocalDateTime end = LocalDateTime.of(LocalDate.now(),LocalTime.MAX);
        lambdaQueryWrapper.between(Attendance::getCreateDate,begin,end);
        return count(lambdaQueryWrapper);
    }

    /**
     * 根据id查询，带关联信息
     * @param request
     * @return
     */
    public AttendanceResponse findByIdWithInfo(AttendanceRequest request){
        return baseMapper.findByIdWithInfo(request);
    }
}