package cn.com.ty.lift.quan.news.service.impl;

import cn.com.ty.lift.quan.news.dao.entity.QuanNews;
import cn.com.ty.lift.quan.news.dao.entity.model.QuanNewsReq;
import cn.com.ty.lift.quan.news.dao.mapper.QuanNewsMapper;
import cn.com.ty.lift.quan.news.service.QuanNewsService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 电梯圈新闻公告 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-02
 */
@Service
@AllArgsConstructor
public class QuanNewsServiceImpl extends ServiceImpl<QuanNewsMapper, QuanNews> implements QuanNewsService {

    private final RedisTemplate<String, Object> redisTemplate;

    /**
     * @description 将新闻置顶
     * @date 2019/12/3 11:23
     */
    public RestResponse top(Long id, Integer type) {
        QuanNews byId = this.getById(id);
        if (byId == null) {
            return RestResponse.success();
        }
        //1置顶 0取消置顶
        if (type == 1) {
            if (byId.getTopSerial() != 0) {
                //如果新闻在置顶新闻中
                Integer serial = byId.getTopSerial();

                List<QuanNews> tops = this.list(new QueryWrapper<QuanNews>().ne("top_serial", 0));
                for (QuanNews top : tops) {
                    if (top.getTopSerial() < serial) {
                        top.setTopSerial(top.getTopSerial() + 1);
                    }
                }
                byId.setTopSerial(1);
                //更新排序
                this.updateBatchById(tops);
                this.updateById(byId);
            } else {
                //如果新闻不是置顶新闻
                //获取置顶新闻
                List<QuanNews> news = this.list(new QueryWrapper<QuanNews>().select("id", "top_serial").ne("top_serial", 0));
                if (!news.isEmpty()) {
                    //如果有置顶新闻
                    news.forEach(n -> {
                        n.setTopSerial(n.getTopSerial() + 1);
                    });
                    this.updateBatchById(news, 100);
                }
                byId.setTopSerial(1);
                //更新置顶新闻列表，新闻
                if (this.updateById(byId)) {
                    return RestResponse.success(null, "置顶成功");
                } else {
                    return RestResponse.fail();
                }
            }
            return RestResponse.success(null, "置顶成功");
        }
        //1置顶 0取消置顶
        if (type == 0) {
            if (byId.getTopSerial() == 0) {
                //如果不是置顶新闻
                return RestResponse.success(null, "不是置顶新闻");
            }
            List<QuanNews> tops = this.list(new QueryWrapper<QuanNews>().select("id", "top_serial").ne("top_serial", 0).ne("id", id));
            //如果置顶新闻为0个
            if (tops.isEmpty()) {
                byId.setTopSerial(0);
                if (this.updateById(byId)) {
                    return RestResponse.success(null, "取消置顶成功");
                } else {
                    return RestResponse.fail();
                }
            }
            //比这个新闻排序大的都 序号-1
            tops.forEach(top -> {
                if (top.getTopSerial() > byId.getTopSerial()) {
                    top.setTopSerial(top.getTopSerial() - 1);
                }
            });
            byId.setTopSerial(0);
            if (this.updateById(byId) && this.updateBatchById(tops)) {
                return RestResponse.success(null, "取消置顶成功");
            } else {
                return RestResponse.fail();
            }
        }
        return RestResponse.failParam();
    }

    /**
     * @description 查询置顶新闻+普通新闻列表
     * @date 2019/12/3 10:30
     */
    public RestResponse news(Long pageNum, Long pageSize, Integer haveTops) {
        Page<QuanNews> page = new Page<>(pageNum, pageSize);
        IPage<Map<String, Object>> news = this.pageMaps(page, new QueryWrapper<QuanNews>().eq("top_serial", "0").orderByDesc("release_date"));
        //全部新闻的浏览量列表
        Map<Object, Object> lookTimes = redisTemplate.boundHashOps("system-quanNews-look-time").entries();
        //全部新闻的点赞用户列表
        Map<Object, Object> likeUsers = redisTemplate.opsForHash().entries("system-quanNews-like-users");
        HashMap<String, Object> result = new HashMap<>();
        newsAddLookAndLike(news.getRecords(), lookTimes, likeUsers);
        result.put("news", news);
        if (haveTops == 1) {
            List<Map<String, Object>> topNews = this.listMaps(new QueryWrapper<QuanNews>().ne("top_serial", "0").orderByAsc("top_serial"));
            //把点赞的用户数量和浏览量整入新闻字段中
            newsAddLookAndLike(topNews, lookTimes, likeUsers);
            result.put("topNews", topNews);
        }
        return RestResponse.success(result);
    }

    //把点赞的用户数量和浏览量整入新闻字段中
    void newsAddLookAndLike(List<Map<String, Object>> records, Map<Object, Object> lookTimes, Map<Object, Object> likeUsers) {
        for (Map<String, Object> record : records) {
            Object id = record.get("id");
            //如果没有浏览量
            if (lookTimes.isEmpty()) {
                record.put("lookNum", "0");
            } else {
                //遍历lookTimes，查看对应id的新闻浏览量
                for (Map.Entry<Object, Object> entry : lookTimes.entrySet()) {
                    Object key = entry.getKey();
                    Object value = entry.getValue();
                    if (key.equals(id.toString())) {
                        record.put("lookNum", value);
                        break;
                    }
                    record.put("lookNum", "0");
                }
            }
            //如果没有用户喜爱
            if (likeUsers.isEmpty()) {
                record.put("likeNum", "0");
            } else {
                //遍历likeUsers，查看对应id的新闻的点赞量
                for (Map.Entry<Object, Object> entry : likeUsers.entrySet()) {
                    String k = entry.getKey().toString();
                    ArrayList v = (ArrayList) entry.getValue();
                    if (k.equals(id.toString())) {
                        record.put("likeNum", v.size());
                        break;
                    }
                    record.put("likeNum", "0");
                }
            }
        }
    }

    public RestResponse addNew(QuanNewsReq req) {
        QuanNews quanNews = new QuanNews();
        quanNews.setTitle(req.getTitle())
                .setContent(req.getContent())
                .setReleaseId(req.getReleaseId())
                .setReleaseUser(req.getReleaseUser());

        List<QuanNews> tops;
        if (req.getIsTop() != 0) {
            //获取置顶新闻列表
            tops = this.list(new QueryWrapper<QuanNews>().ne("top_serial", 0));
            //设置新添加新闻的数据
            quanNews.setTopSerial(1);
            //遍历排序加1
            if (!tops.isEmpty()) {
                tops.forEach(top -> {
                    top.setTopSerial(top.getTopSerial() + 1);
                });
                if (this.updateBatchById(tops) && this.save(quanNews)) {
                    return RestResponse.success(quanNews.getId(), "成功");
                } else {
                    return RestResponse.fail();
                }
            } else {
                if (this.save(quanNews)) {
                    return RestResponse.success(quanNews.getId(), "成功");
                } else {
                    return RestResponse.fail();
                }
            }
        }
        //添加新闻
        if (this.save(quanNews)) {
            return RestResponse.success(quanNews.getId(), "成功");
        } else {
            return RestResponse.fail();
        }
    }

    /**
     * @description 更新新闻
     * @date 2019/12/4 13:34
     */
    public RestResponse updateNews(QuanNews quanNews) {
        if (this.updateById(quanNews)) {
            return RestResponse.success(null, "成功");
        } else {
            return RestResponse.fail();
        }
    }

    /**
     * @description 删除新闻
     * @date 2019/12/4 13:55
     */
    public RestResponse delete(Long id) {
        QuanNews byId = this.getById(id);
        if (null == byId) {
            return RestResponse.success();

        }
        if (byId.getTopSerial() != 0) {
            //取消新闻置顶
            this.top(id, 0);
        }
        if (this.removeById(id)) {
            return RestResponse.success(null, "成功");
        } else {
            return RestResponse.fail();
        }
    }

    public RestResponse like(QuanNewsReq req) {
        String uid = req.getUserId().toString();
        String id = req.getId().toString();
        ArrayList users = (ArrayList) redisTemplate.opsForHash().get("system-quanNews-like-users", id);
        if (users == null) {
            users = new ArrayList<>();
            users.add(uid);
            redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
            return RestResponse.success(null, "点赞成功");
        } else {
            //遍历userId列表
            for (Object user : users) {
                if (user.equals(uid)) {
                    //如果列表内有这个uid就删除喜欢,并return
                    users.remove(user);
                    redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
                    return RestResponse.success(null, "取消点赞成功");
                }
            }
            //如果列表内没有这个uid就标记喜欢
            users.add(uid);
            redisTemplate.opsForHash().put("system-quanNews-like-users", id, users);
            return RestResponse.success(null, "点赞成功");
        }
    }

    //获取一条新闻
    public RestResponse getQuanNews(QuanNewsReq req) {
        Map<String, Object> byId = this.getMap(new QueryWrapper<QuanNews>().eq("id", req.getId()));
        if (null != byId) {
            String id = byId.get("id").toString();
            Long lookNum = redisTemplate.boundHashOps("system-quanNews-look-time").increment(id, 1);
            int isLike;
            ArrayList users = (ArrayList) redisTemplate.opsForHash().get("system-quanNews-like-users", id);
            if (users != null && users.contains(req.getUserId().toString())) {
                isLike = 1;
            } else {
                isLike = 0;
            }
            byId.put("lookNum", lookNum);
            byId.put("isLike", isLike);
            return RestResponse.success(byId, "成功");
        } else {
            return RestResponse.success();
        }
    }
}