package cn.com.ty.lift.system.settings.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.utils.ProjectUtils;
import cn.com.ty.lift.system.settings.dao.entity.MaintenanceCompany;
import cn.com.ty.lift.system.settings.dao.entity.MtCompanyAttestation;
import cn.com.ty.lift.system.settings.dao.entity.model.MaintenanceCompanyRequest;
import cn.com.ty.lift.system.settings.dao.mapper.MaintenanceCompanyMapper;
import cn.com.ty.lift.system.settings.service.IMaintenanceCompanyService;
import cn.com.ty.lift.system.settings.service.IMtCompanyAttestationService;
import cn.com.ty.lift.system.user.dao.entity.MtCompanyUser;
import cn.com.ty.lift.system.user.dao.entity.Role;
import cn.com.ty.lift.system.user.dao.entity.UserAccount;
import cn.com.ty.lift.system.user.dao.entity.UserRole;
import cn.com.ty.lift.system.user.dao.entity.model.MenuRequest;
import cn.com.ty.lift.system.user.service.*;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 维保公司 服务实现类
 * </p>
 *
 * @author huangyuan
 * @since 2019-12-09
 */
@Service
public class MaintenanceCompanyServiceImpl extends ServiceImpl<MaintenanceCompanyMapper, MaintenanceCompany> implements IMaintenanceCompanyService {

    @Resource
    private IRoleService roleService;

    @Resource
    private IUserRoleService userRoleService;

    @Resource
    private IMenuService menuService;

    @Resource
    private IUserAccountService userAccountService;

    @Resource
    private IMtCompanyUserService mtCompanyUserService;

    @Resource
    private IMtCompanyAttestationService mtCompanyAttestationService;

    @Override
    public RestResponse list(MaintenanceCompanyRequest maintenanceCompanyRequest) {
        IPage<MaintenanceCompany> maintenanceCompanyPage = new Page<>(
                maintenanceCompanyRequest.getPageNum(), maintenanceCompanyRequest.getPageSize());
        maintenanceCompanyPage = this.page(
                maintenanceCompanyPage, new QueryWrapper<MaintenanceCompany>()
                        //通过公司名称模糊查找公司信息
                        .like(StringUtils.isNotBlank(maintenanceCompanyRequest.getCompanyName()), "name", maintenanceCompanyRequest.getCompanyName())
        );
        //设置维保公司人数
        setCompanyUserNum(maintenanceCompanyPage.getRecords());
        return RestResponse.success(maintenanceCompanyPage, ApiConstants.RESULT_SUCCESS, "获取公司列表成功");
    }

    @Override
    @Transactional
    public RestResponse createTeam(MaintenanceCompanyRequest maintenanceCompanyRequest) {
        MaintenanceCompany maintenanceCompany = new MaintenanceCompany();
        //当前登录用户id
        Long userId = maintenanceCompanyRequest.getUserId();
        //设置申请团队信息
        maintenanceCompany.setName(maintenanceCompanyRequest.getCompanyName());
        maintenanceCompany.setCorporator(maintenanceCompanyRequest.getCorporator());
        maintenanceCompany.setCreatorId(userId);
        //设置团队状态为未认证
        maintenanceCompany.setIsCertificated(ApiConstants.CompanyConstants.MAINTENANCE_NOT_CERTIFICATE);
        maintenanceCompany.setRemarks(maintenanceCompanyRequest.getRemarks());
        maintenanceCompany.setLogoImg(maintenanceCompanyRequest.getLogoImg());
        this.save(maintenanceCompany);
        //公司id
        Long companyId = maintenanceCompany.getId();

        //创建团队成功
        //1.设置申请用户为企业管理员
        UserAccount userAccount = userAccountService.getByUserId(userId);
        userAccount.setType(ApiConstants.UserConstants.TYPE_ENTERPRISE_ADMIN);
        userAccountService.updateById(userAccount);
        //2.设置用户类型为企业管理员
        MtCompanyUser mtCompanyUser = new MtCompanyUser();
        mtCompanyUser.setCreateUserId(userId);
        mtCompanyUser.setUserId(userId);
        mtCompanyUser.setMtCompanyId(companyId);
        mtCompanyUserService.save(mtCompanyUser);
        //3.创建默认角色-企业管理员
        Role role = new Role();
        role.setName(CommonEnum.DefaultRole.ENTERPRISE_ADMIN.getLabel());
        role.setCompanyId(companyId);
        role.setDescription("企业管理员");
        roleService.save(role);
        //4.创建用户角色关联关系
        UserRole userRole = new UserRole();
        userRole.setUserId(userId);
        userRole.setRoleId(role.getId());
        userRole.setCompanyId(companyId);
        userRoleService.save(userRole);

        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "创建团队成功");
    }

    @Override
    @Transactional
    public RestResponse updateMaintenanceCompany(MaintenanceCompanyRequest maintenanceCompanyRequest) {
        //获取申请消息
        MtCompanyAttestation mtCompanyAttestation = mtCompanyAttestationService.getById(maintenanceCompanyRequest.getId());
        //获取公司id
        Long companyId = mtCompanyAttestation.getMtCompanyId();
        MaintenanceCompany maintenanceCompany = this.getById(companyId);
        //更新vip标识
        maintenanceCompany.setVipFlag(maintenanceCompanyRequest.getVipFlag());
        //更新电梯台量
        maintenanceCompany.setLimitedNum(maintenanceCompanyRequest.getLimitedNum());
        //更新企业菜单信息
        menuService.companyMenu(new MenuRequest(companyId, maintenanceCompanyRequest.getMenuIds()));
        this.updateById(maintenanceCompany);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "更新公司信息成功");
    }

    @Override
    public RestResponse ownList(MaintenanceCompanyRequest maintenanceCompanyRequest) {
        //获取关联关系
        List<MtCompanyUser> mtCompanyUserList = mtCompanyUserService.list(new QueryWrapper<MtCompanyUser>()
                .eq("user_id", maintenanceCompanyRequest.getUserId())
        );
        //获取所在的团队信息
        if (mtCompanyUserList != null && mtCompanyUserList.size() > 0) {
            List<Long> companyIdList = ProjectUtils.getAttrList(mtCompanyUserList, "mtCompanyId", null);
            List<MaintenanceCompany> maintenanceCompanyList = (List<MaintenanceCompany>) this.listByIds(companyIdList);
            //设置维保公司人数
            setCompanyUserNum(maintenanceCompanyList);
            for(MaintenanceCompany maintenanceCompany : maintenanceCompanyList) {
                if(maintenanceCompany.getId() == maintenanceCompanyRequest.getCompanyId()) {
                    maintenanceCompany.setCurrentTeamFlag(true);
                }
            }
            return RestResponse.success(maintenanceCompanyList, ApiConstants.RESULT_SUCCESS, "获取用户所在团队列表成功");
        }
        return RestResponse.success(null, ApiConstants.RESULT_NO_DATA, "用户还未加入任何团队");
    }

    @Override
    public void setCompanyUserNum(List<MaintenanceCompany> maintenanceCompanyList) {
        if (maintenanceCompanyList != null && maintenanceCompanyList.size() > 0) {
            List<Long> companyId = ProjectUtils.getAttrList(maintenanceCompanyList, "id", null);
            //获取公司成员信息
            List<MtCompanyUser> mtCompanyUserList = mtCompanyUserService.list(new QueryWrapper<MtCompanyUser>()
                    .select("mt_company_id")
                    .in("mt_company_id", companyId)
            );
            Map<Long, List<MtCompanyUser>> companyIdToMtCompanyUser = ProjectUtils.attrToListMap(mtCompanyUserList, "mtCompanyId", null);
            //循环设置公司人数
            for (MaintenanceCompany maintenanceCompany : maintenanceCompanyList) {
                List<MtCompanyUser> mtCompanyUsers = companyIdToMtCompanyUser.get(maintenanceCompany.getId());
                maintenanceCompany.setUserNum(0);
                if (mtCompanyUsers != null) {
                    maintenanceCompany.setUserNum(mtCompanyUsers.size());
                }
            }
        }
    }

}
